/*
Purpose: Retrieves Pokémon stats from memory and displays them on the bottom
screen. Shows both Wild Pokémon and Party Pokémon stats. Each can be toggled
on or off with L & R
*/

/*
12a13
> #define __dv (*(vu16*)0x0892fd19)
113c114
< xsprintf(buf, "%02d:%02d", hour, min);
---
> xsprintf(buf, "%04X", __dv);

Original code by shadowofdarkness

Gba Temp Thread
https://gbatemp.net/threads/release-gen-1-pokemon-vc-wild-encounter-dv-ntr-overlay.459869/

v0.8.6-B
By Veund

Additions and formatting by AnalogMan
*/

#include "global.h"
#include "ov.h"

FS_archive sdmcArchive = { 0x9, (FS_path){ PATH_EMPTY, 1, (u8*)"" } };
Handle fsUserHandle = 0;
Result APT_CheckNew3DS(bool* out);
void initSharedFunc(void);

#define CALLBACK_OVERLAY (101)

#define TICKS_PER_MSEC (268123.480)

#define WHITE 0xFFFFFF
#define RED   0xFF0000
#define GREEN 0x00FF00
#define GOLD  0xD4AF37


#define __ver (*(vu16*)0x0880014b) /*Game Version Location variable*/

/*d057*/
#define __bbat  (*(vu8*)0x0892FDB4) /*Pokemon Blue Battle Detection*/
#define __rbat  (*(vu8*)0x0892FDB4) /*Pokemon Red Battle Detection*/
#define __ybat  (*(vu8*)0x08930917) /*Pokemon Yellow Battle Detection*/
#define __gbat  (*(vu8*)0x088AFC1C) /*Pokemon Green Battle Detection*/
#define __sbat  (*(vu8*)0x08A2F07A) /*Pokemon Gold/Silver Battle Detection*/
#define __cbat  (*(vu8*)0x08A2F325) /*Pokemon Crystal Battle Detection*/

/*CF94*/
#define __bpar  (*(vu8*)0x0892FCF1) /*Pokemon Blue Status Screen Detection*/
#define __rpar  (*(vu8*)0x0892FCF1) /*Pokemon Red Status Screen Detection*/
#define __ypar  (*(vu8*)0x08930854) /*Pokemon Yellow Status Screen Detection*/
#define __gpar  (*(vu8*)0x088AFB63) /*Pokemon Green Status Screen Detection*/


/* Doesn't work the same as Gen I. Should report 0 in battle, 1 on status screen
and 2 on Pokémon menu. Fails to report 2 on Pokémon menu and retains value
of 1 until battle is entered */
#define __spar  (*(vu8*)0x08A2EF57) /*Pokemon Gold/Silver Status Screen Detection*/
#define __cpar  (*(vu8*)0x08A2F202) /*Pokemon Crystal Status Screen Detection*/

/*cff1*/
#define __dvbw  (*(vu16*)0x0892fd4e) /*Pokemon Blue DV location for wild battle*/
#define __dvrw  (*(vu16*)0x0892fd4e) /*Pokemon Red DV location for wild battle*/
#define __dvyw  (*(vu16*)0x089308b1) /*Pokemon Yellow DV location for wild battle*/
#define __dvgw  (*(vu16*)0x088afbc0) /*Pokemon Green DV location for wild battle*/
#define __dvsw  (*(vu16*)0x08A2F059) /*Pokemon Gold/Silver DV location for wild battle*/
#define __dvcw  (*(vu16*)0x08A2F304) /*Pokemon Crystal DV location for wild battle*/

/*cfb3*/
#define __dvbp  (*(vu16*)0x0892FD10) /*Pokemon Blue DV location for party*/
#define __dvrp  (*(vu16*)0x0892FD10) /*Pokemon Red DV location for party*/
#define __dvyp  (*(vu16*)0x08930873) /*Pokemon Yellow DV location for party*/
#define __dvgp  (*(vu16*)0x088AFB82) /*Pokemon Green DV location for party*/
#define __dvsp  (*(vu16*)0x08A2F9A3) /*Pokemon Gold/Silver DV location for party*/
#define __dvcp  (*(vu16*)0x08A2FDEC) /*Pokemon Crystal DV location for party*/

#define __dvse  (*(vu16*)0x08A2FC3F) /*Pokemon Gold/Silver DV location for daycare egg*/
#define __dvce  (*(vu16*)0x08A30088) /*Pokemon Crystal DV location for daycare egg*/

#define __segg  (*(vu8*)0x8A2F7F7) /*Pokemon Gold/Silver egg available at daycare*/
#define __cegg  (*(vu8*)0x8A2F8C0) /*Pokemon Crystal egg available at daycare*/


char hiddenName[][16] = {"Fighting", "Flying", "Poison", "Ground", "Rock", "Bug", "Ghost", "Steel", "Fire", "Water", "Grass", "Electric", "Psychic", "Ice", "Dragon", "Dark"}; /*Hidden Power Types*/
int hiddenColour[16] = {0xC9656F, 0x71A5FB, 0x9787B8, 0xD3A35A, 0x928167, 0x95AD3F, 0x4D5F9D, 0xC0C0C0, 0xEE0B0B, 0x4045FF, 0x4E9131, 0xFBF259, 0x8230B8, 0x82FFE6, 0x3540BD, 0x4A4A4A }; /*Type colours extracted from a crystal Sprite map *in folder* */
/*							0		 1		2			3		4			5		6			7		8			9		10			11		12			13		14		  15			helps with hidden power debug		*/


u32 IoBasePad;
bool togglePressed = false,
     gen1 = false,
     gen2 = false;

/* Initialize display of overlay boxes to off */
bool partyOn = false,
     wildOn = false,
     eggOn = false;

unsigned int battle = 0,
	     egga = 0,
             party = 0;

 /*
 DV Calculator
 dv: raw hex value of stats to be calculated
 x: X position on screen box starts at in pixels
 y: Y position on screen box starts at in pixels
 w: width of box in pixels
 h: height of box in pixels
 */
void dvCalc(unsigned int dv, int x, int y, int w, int h)
{
    char buf[60];

    /* Break DV value up into individual stats and calculate HP stat */
    unsigned int speed = (dv >> 0xC) & 0xF,
                 special = (dv >> 0x8) & 0xF,
                 attack = (dv >> 0x4) & 0xF,
                 defense = dv & 0xF,
                 hp = ((attack & 0x1) << 3) + ((defense & 0x1) << 2) + ((speed & 0x1) << 1) + (special & 0x1);
	unsigned int hidden = ((attack & 3) << 2) + (defense & 3);
	

    /* Draw transparent box with variable position and size */
    OvDrawTranspartBlackRect(x, y, w, h, 1);

    /* Write HP line and color code according to value and a position relative to box */
    xsprintf(buf, "%-12s %2d", "HP:", hp);
    OvDrawString(buf, x+1, y+14, (hp == 15) ? GREEN : (hp == 0) ? RED : WHITE);

    /* Write Attack line and color code according to value and a position relative to box */
    xsprintf(buf, "%-12s %2d", "Attack:", attack);
    OvDrawString(buf, x+1, y+25, (attack == 15) ? GREEN : (attack == 0) ? RED : WHITE);

    /* Write Defense line and color code according to value and a position relative to box */
    xsprintf(buf, "%-12s %2d", "Defense:", defense);
    OvDrawString(buf, x+1, y+36, (defense == 15) ? GREEN : (defense == 0) ? RED : WHITE);

    /* Write Speed line and color code according to value and a position relative to box */
    xsprintf(buf, "%-12s %2d", "Speed:", speed);
    OvDrawString(buf, x+1, y+47, (speed == 15) ? GREEN : (speed == 0) ? RED : WHITE);

    /* Write Special line and color code according to value and a position relative to box */
    xsprintf(buf, "%-12s %2d", "Special:", special);
    OvDrawString(buf, x+1, y+58, (special == 15) ? GREEN : (special == 0) ? RED : WHITE);
	
	/* Write Hidden Power type line and color code according to value and a position relative to box */
	xsprintf(buf,"HP:");
	OvDrawString(buf, x+1, y+87, WHITE); 
	xsprintf(buf, hiddenName[hidden]);
	OvDrawString(buf, x+22, y+87, hiddenColour[hidden]); 
	
	/* debug Hidden Power code*/
	/*    xsprintf(buf, "%-12s %2d", "HP debug:", hidden);;
        OvDrawString(buf, x+1, y+98, GOLD);*/

    /*
    Compares DV value to known values that result in a shiny
    when transferred and outputs a position relative string
    if a match is found
    */
    if (dv == 0xAAAA) {
        xsprintf(buf, "Universal Shiny");
        OvDrawString(buf, x+1, y+74, GOLD);

    } else if ((dv & 0xFFF2) == 0xAAA2) {
        xsprintf(buf, "Gen 7 Shiny");
        OvDrawString(buf, x+1, y+74, GOLD);

    } else if ((dv & 0xFF2F) == 0xAA2A) {
        xsprintf(buf, "Gen 2 Shiny");
        OvDrawString(buf, x+1, y+74, GOLD);

    } else {
        xsprintf(buf, "Not Shiny");
        OvDrawString(buf, x+1, y+74, WHITE);
    }
}

/*
Determines which game is loaded and assigns stat value from appropriate address.
Checks if overlays are active and calls the ones that are, then writes title of
overlay inside of box.
*/
void drawWidget(void)
{
    unsigned int dvw, dvp, dve;

    switch(__ver) {
        case 0x4220: // Blue
            dvw = __dvbw;
            dvp = __dvbp;
            battle = __bbat;
            party = __bpar;
            gen1 = true;
            break;
        case 0x5220: // Red
            dvw = __dvrw;
            dvp = __dvrp;
            battle = __rbat;
            party = __rpar;
            gen1 = true;
            break;
        case 0x5920: // Yellow
            dvw = __dvyw;
            dvp = __dvyp;
            battle = __ybat;
            party = __ypar;
            gen1 = true;
            break;
		case 0x4720: // Green
            dvw = __dvgw;
            dvp = __dvgp;
            battle = __gbat;
            party = __gpar;
            gen1 = true;
            break;
        case 0x535F: // Silver
            dvw = __dvsw;
            dvp = __dvsp;
            dve = __dvse;
            egga = __segg; 
            battle = __sbat;
            party = __spar;
            gen2 = true;
            break;
        case 0x475F: // Gold
            dvw = __dvsw;
            dvp = __dvsp;
            dve = __dvse;
            egga = __segg;
            battle = __sbat;
            party = __spar;
            gen2 = true;
            break;
        case 0x4154: // Crystal
            dvw = __dvcw;
            dvp = __dvcp;
            dve = __dvce;
            egga = __cegg;
            battle = __cbat;
            party = __cpar;
            gen2 = true;
            break;
        default:
            return;   /* If no match is found, exit function without doing anything */
    }

    unsigned int testDV1 = 0xAAAA, /* Universal Shiny */
                 testDV2 = 0xAA3A, /* Gen 2 Shiny */
                 testDV3 = 0xAAAF, /* Gen 7 Shiny */
                 testDV4 = 0xFFFF; /* Perfect DV, Not Shiny */


    if (wildOn || battle == 1) {
        dvCalc(dvw, 14, 9, 92, 99);
        OvDrawString("Wild Pokemon", 14+1, 9+3, WHITE);
    }
    if ((partyOn || party == 1) && !battle) {
        dvCalc(dvp, 214, 9, 92, 99);
        OvDrawString("Party Pokemon", 214+1, 9+3, WHITE);
    }
    if (eggOn && gen2 && !battle) {
        dvCalc(dve, 114, 9, 92, 99);
        OvDrawString("Unclaimed Egg", 114+1, 9+3, WHITE);
		if ((egga & 0b00100000) != 0b0 && gen2 && eggOn) {
          OvDrawString("(!)", 114+75, 95+3, GREEN);
		}
	
	
	}
    char buf[40];
    // xsprintf(buf, "Debug: 0x%02X", party);
    // OvDrawString(buf, 115, 230, WHITE);
}

/* Returns currently pressed key */
u32 getKey()
{
    return (*(vu32*)(IoBasePad) ^ 0xFFF) & 0xFFF;
}

/*
Checks current key, then checks if button has been released since last check.
If it has, check is cleared and the next button press is processed. Toggles
overlays on or off when pressing L & R
*/
void handleKey(void) {
    u32 curkey = getKey();
    if(togglePressed) {
        if(curkey == 0)
            togglePressed = false;
    } else {
        switch(curkey) {
            case BUTTON_L:
                wildOn = !wildOn;
                togglePressed = true;
                break;
            case BUTTON_R:
                partyOn = !partyOn;
                togglePressed = true;
                break;
            case BUTTON_Y:
                eggOn = !eggOn;
                togglePressed = true;
                break;
        }
    }
}

/*
Overlay Callback.
isBottom: 1 for bottom screen, 0 for top screen.
addr: writable cached framebuffer virtual address, should flush data cache after modifying.
addrB: right-eye framebuffer for top screen, undefined for bottom screen.
stride: framebuffer stride(pitch) in bytes, at least 240*bytes_per_pixel.
format: framebuffer format, see https://www.3dbrew.org/wiki/GPU/External_Registers for details.

return 0 on success. return 1 when nothing in framebuffer was modified.
*/

u32 overlayCallback(u32 isBottom, u32 addr, u32 addrB, u32 stride, u32 format)
{
    OvSettings(addr, addrB, stride, format, isBottom);
    handleKey();

    /* Only draws overlays on bottom screen */
    if (isBottom)
        drawWidget();
    return 0;
}

/*
Checks if n3DS or o3DS, then initializes key capture.
Calls Overlay Callback
*/
int main()
{
    initSharedFunc();
    initSrv();

    bool isNew3DS = false;
    APT_CheckNew3DS(&isNew3DS);

    if (isNew3DS)
        IoBasePad = 0xFFFC2000;
    else
        IoBasePad = 0xFFFC6000;

    plgRegisterCallback(CALLBACK_OVERLAY, (void*) overlayCallback, 0);
    return 0;
}
